"""OOPS Core Types.
"""
from __future__ import generators
from types import LambdaType, ClassType
from bsddb import hashopen
from cPickle import loads, dumps
from weakref import ref
from math import sqrt


bottle = dumps
unbottle = loads


class Node(object):
	"""Persistent Container (base class to all of OOPS).

	Nodes represent positions in a dictionary-like database. A tree structure
	of nodes (like a tree structure of instances) is mirrored on the database
	by using each node's path as its identifying key. An example of this is if
	you have an node that you can reach in python by typing parent.node.child,
	its path (stored in its __path__ attribute) is the tuple ('node', 'child').
	The reason the path does not have the root object's key is that is was not
	necessary for me to store it (and also it is currently impossible using
	python's standard hooks to get that name).

	If a node is bound (already has a parent node) and is assigned to a new
	container node, a copy of it is made and that copy is bound instead.

	Example::

		>>> root = Node()
		>>> print root.__path__
		()
		>>> root.subnode = Node() # bind Node instance to root
		>>> print root.subnode.__path__
		('subnode',)
		>>> newnode = Node() # unbound
		>>> print newnode.__path__
		()
		>>> root.subnode.anothernode = newnode # bind to root.subnode
		>>> print root.subnode.anothernode.__path__
		('subnode', 'anothernode')
		>>> root.copynode = root.subnode
		>>> print root.copynode.__path__, root.subnode.__path__
		('copynode',) ('subnode',)

	Node was not designed to be instantiated and used, but to be inherited from
	to create your own persistent classes (two examples of which are Dict and
	List).

	Important attributes:

		__parent__  --  weak reference to the parent node. When a node is
						unbound, this returns None.

		__path__	--  address of this Node in its filesystem. It is a tuple
						of keys identifying each node inside its parent and
						placed in order from the top-most node to the current
						node. example: obj.subobj.newobj.__path__ is
						('subobj', 'newobj')

		__database__	--  dictionary-like object used for persistent storage.
							a repr()'d version of the node's path is used as
							the node's database key.


	* NOTE* You will notice that I follow the general Python naming convention
	for attributes that define behavior (__varname__). The reason for this is
	I don't want those attributes to be out of reach for developers of
	extension libraries, but at the same time I don't want developers that use
	those libraries to be burdened with the implementation details of OOPS.
	"""
	__parent__ = lambda:None
	__parent__ = staticmethod(__parent__)
	__path__ = ()
	def __init__(self, **keywords):
		"""Constructor.

		Optional keyword arguments:

			database	--  initial database to.

			filename	--  binds the node to this file. If the file doesn't
							exist, it is created.

			datasource  --  tuple of initial arguments to __bindto__.

			copyfrom	--  initial argument to __copyfrom__.
		"""
		object.__init__(self)
		vars(self).setdefault("__nodekeys__", [])
		if "datasource" in keywords:
			apply(self.__bindto__, keywords["datasource"])
		elif "filename" in keywords:
			self.__bindto__(hashopen(keywords["filename"], "c"))
		elif "database" in keywords:
			self.__bindto__(keywords["database"])
		else:
			self.__bindto__({})
		if "copyfrom" in keywords:
			self.__copyfrom__(keywords["copyfrom"])

	def __repr__(self):
		"""Representation of a Node instance.

		Overloaded to provide an informative description of a Node instance.
		"""
		return "<%s path=%r>" % (type(self).__name__, self.__path__)

	def __nonzero__(self):
		"""Test for emptiness.

		If the node has row data (or a subnode has row data) the node is
		nonzero.
		"""
		for key in self.__nodekeys__:
			if getattr(self, key):
				return 1
		return 0

	__getstate__ = lambda self: None
	__setstate__ = lambda self, value: None

	def __setattr__(self, name, value):
		"""Attribute assignment operator.

		If value is a node and unbound, it is bound to self. If it is nonzero
		(not empty), it is moved to self. If it is bound, a copy of it is made
		and that copy is either bound or moved depending on if the node is
		empty or not.

		If the attribute by that name is a node, value is assigned to the node
		by calling its __setval__ method.

		If neither value nor the attribute identified by name are nodes, then
		it is a normal attribute assignment.
		"""
		if name in self.__nodekeys__:
			return getattr(self, name).__setval__(value)
		if isinstance(value, Node):
			if value.__parent__() is None:
				if value:
					value.__moveto__(self.__database__,
											self.__path__+(name,), self)
				else:
					value.__bindto__(self.__database__,
											self.__path__+(name,), self)
			else:
				value = type(value)(copyfrom=value,
			datasource=(self.__database__, self.__path__+(name,), self))
			if name not in self.__nodekeys__:
				self.__nodekeys__.append(name)
		object.__setattr__(self, name, value)

	def __delattr__(self, name):
		"""Attribute deletion operator.

		If the attribute identified by name is a node, all its data is
		recursively deleted before finally removing it from the instance's
		attribute dictionary.

		If it is not a node, a regular attribute deletion is executed.
		"""
		if name in self.__nodekeys__:
			getattr(self, name).__clear__()
			self.__nodekeys__.remove(name)
		object.__delattr__(self, name)

	# OOPS behavior methods
	def __bindto__(self, database, path=(), parent=None):
		"""Bind node to new location.

		Recursively bind all subnodes to the new location & load persistent
		runtime data.

		This method is called by the parent node at binding time and also in
		Node.__init__.

			path	--  tuple representing the node's address in the database.

			database	--  the database used to store the node's persistent
							data.

			parent  --  parent node.
		"""
		self.__database__ = database
		if len(path)>0:
			self.__path__ = path
		if parent is not None:
			self.__parent__ = ref(parent)
		for key in self.__nodekeys__:
			getattr(self, key).__bindto__(database, path+(key,), self)

	def __moveto__(self, database, path=(), parent=None):
		"""Move node to new location

		Recursively move all subnodes to the new location.

		This method is called by the parent node at binding time if the node
		has data.

			path	--  tuple representing the node's address in the database.

			database	--  the database used to store the node's persistent
							data.

			parent  --  parent node.
		"""
		self.__database__ = database
		if len(path)>0:
			self.__path__ = path
		if parent is not None:
			self.__parent__ = ref(parent)
		for key in self.__nodekeys__:
			getattr(self, key).__moveto__(database, path+(key,), self)

	def __setval__(self, value):
		"""Assign value to this node.

		This method is not used by Node. It is meant to be overloaded by a
		subclass (for example see List & Dict which both use __setval__ to
		accept regular Python lists and dictionaries respectively).

		If the value is not an instance of the same class as self, it raises
		a RuntimeError.
		"""
		if type(value) is type(self):
			return self.__copyfrom__(value)
		raise RuntimeError("You cannot assign a value to %s instance" %
														type(self).__name__)

	def __copyfrom__(self, other):
		"""Make `self` a copy of `other`.

		This method does not actually do anything, but is there as a hook
		for subclasses.
		"""
		self.__clear__()
		for key in other.__nodekeys__:
			setattr(self, key, getattr(other, key))

	def __clear__(self):
		"""Recursively delete all persistent data under `self`.

		This method does not actually do anything, but is there as a hook
		for subclasses.
		"""
		for key in self.__nodekeys__:
			getattr(self, key).__clear__()


class Keyspace(list, Node):
	"""Keyspace manager.

	Keyspace's role is to keep track of a container node's persistent members.
	In the case of a Dict instance, it is to keep track of the dictionary's
	keys. In the case of an Obj instance, it is to keep track of its data rows.

	Keyspace is specially designed to scale for containers with many subnodes.
	Keyspace instances distribute their contents across many database rows to
	counter the overhead of pickling large lists. The reason for this is that
	the keyspace needs to be synced to disk every time it is modified. With
	this implementation, the keyspace is broken into chunks and only the
	modified chunks are pickled and saved when new keys are added.

	Keyspace is ultra-light on storage size, I/O and processing because it
	uses what I call the square key distribution algorithm. The rows containing
	the distributed keyspace are always smaller than the square root of the
	size of the keyspace. There are usually as many rows as there are elements
	per row, but when a new square size is reached (len(keyspace)==x**2), new
	elements are placed in a new row (so rows==row_size+1).

	When adding an element, only one row is synced to disk which has at most
	row_size+1 elements. When removing an element, at most 2 rows are synced to
	disk while sometimes only one row is synced to disk.
	"""
	def __init__(self, **keywords):
		"""Constructor.

		Overloaded to call list.__init__
		"""
		list.__init__(self)
		Node.__init__(self, **keywords)

	def __nonzero__(self):
		"""Test for emptiness.

		Returns true (1) if the keyspace has one key or more, false (0)
		otherwise.
		"""
		return len(self)>0

	def add_key(self, key):
		"""Insert a key into the keyspace.

		If you insert a key that is already in the keyspace, the function
		returns without doing anything.
		"""
		if key in self:
			return
		chunk = int(sqrt(len(self)))
		row = len(self) - (chunk+1)*chunk
		if row<0:
			self.append(key)
			self.__database__[`self.__path__+(chunk,)`] = \
														bottle(self[chunk**2:])
		else:
			self.insert(row*(chunk+1), key)
			self.__database__[`self.__path__+(row,)`] = \
								bottle(self[row*(chunk+1):(row+1)*(chunk+1)])

	def rem_key(self, key):
		"""Remove a key from the keyspace.

		Raises a KeyError is the key is not in the keyspace.
		"""
		try:
			dead_item_pos = self.index(key)
		except ValueError:
			raise KeyError(key)
		row_size = int(sqrt(len(self)))
		oversized_rows = max([len(self)-(row_size+1)*row_size, 0])
		dead_item_row = (dead_item_pos - min([oversized_rows,
								(dead_item_pos+1)/(row_size+1)])) / row_size
		if oversized_rows:
			last_item_row = oversized_rows-1
		else:
			last_item_row = row_size - (row_size**2==len(self))
		last_item_row_start = last_item_row*row_size + \
										min([oversized_rows, last_item_row])
		last_item_row_until = last_item_row_start + row_size + \
											(oversized_rows-1>last_item_row)
		if dead_item_row!=last_item_row:
			dead_item_row_start = dead_item_row*row_size + \
										min([oversized_rows, dead_item_row])
			dead_item_row_until = dead_item_row_start + row_size + \
											(oversized_rows-1>dead_item_row)
			self[dead_item_pos] = self[last_item_row_start]
			self.__database__[`self.__path__+(dead_item_row,)`] = \
						bottle(self[dead_item_row_start:dead_item_row_until])
			del self[last_item_row_start]
		else:
			del self[dead_item_pos]
		if len(self) == row_size**2:
			del self.__database__[`self.__path__+(last_item_row,)`]
		else:
			self.__database__[`self.__path__+(last_item_row,)`] = \
						bottle(self[last_item_row_start:last_item_row_until])

	def __bindto__(self, database, path=(), parent=None):
		"""Bind node to new location.

		Overloaded to load reconstitute the keyspace from chunks stored on
		disk.
		"""
		Node.__bindto__(self, database, path=(), parent=None)
		del self[:]
		at_row = 0
		while 1:
			try:
				self.extend(
						unbottle(self.__database__[`self.__path__+(at_row,)`]))
			except KeyError:
				break
			at_row += 1

	def __moveto__(self, database, path=(), parent=None):
		"""Move node to new location.

		Overloaded to copy the keyspace chunks to the new database location.
		"""
		at_row = 0
		while 1:
			try:
				database[`path+(at_row,)`] = \
								self.__database__[`self.__path__+(at_row,)`]
			except KeyError:
				break
			at_row += 1
		Node.__moveto__(self, database, path=(), parent=None)

	def __copyfrom__(self, other):
		"""Make `self` a copy of `other`

		Overloaded to copy over all the keyspace's data.
		"""
		Node.__copyfrom__(self, other)
		at_row = 0
		while 1:
			try:
				self.__database__[`self.__path__+(at_row,)`] = \
								other.__database__[`other.__path__+(at_row,)`]
			except KeyError:
				break
			at_row += 1
		self[:] = other

	def __clear__(self):
		"""Recursively delete all persistent data under `self`.

		Overloaded to empty the keyspace.
		"""
		Node.__clear__(self)
		del self[:]
		at_row = 0
		while 1:
			try:
				del self.__database__[`self.__path__+(at_row,)`]
			except KeyError:
				break
			at_row += 1


class Unit(object):
	"""Persistent attribute (base class for all units).

	Unit classes implement the property hooks introduced in Python 2.2 for
	class attributes that intercept operations on instance attributes. This
	allows units in an Obj subclass' definition to automatically do type
	checking and store attribute values persistently, transparently.

	Unit is not really meant to be used as-is (although it can be). It is meant
	to be a minimalist base class for other units to inherit (examples of such
	classes include Data/String, Text/Unicode, Int, Float, Type, Link and Date)

	Important Attributes:

		default	--	callable object that returns the value to return if the
					unit's data row is empty. The default default value is
					None.

	"""
	constraint = lambda self, value: 1 # no constraint: always return true
	def __init__(self, *default_value, **keywords):
		"""Constructor

		Arguments:

			default_value	--	if specified, it is used to build a lambda that
								returns it as a default value.
		Keyword Arguments:

			default	--	expected to be a callable object that returns the
						default value for this unit.

			constraint	--	a callable object that returns true (1) if the
							first argument is a valid value for this field,
							false (0) if it isn't.
		"""
		object.__init__(self)
		if "default" in keywords:
			self.default = keywords["default"]
		elif default_value:
			self.default = lambda val=default_value[0]: val
		if "constraint" in keywords:
			self.constraint = keyword["constraint"]

	def default(self):
		"""Default value to return if the database contains no data in this
		unit's row.

		Return None.
		"""
		return None

	def validate(self, value):
		"""Validate value.

		Calls the constraint attribute to make sure this value doesn't
		contradict it.

		By default this always returns true (1), but it this method can be
		overloaded to return false (0) is the value isn't valid for this
		type.
		"""
		if not self.constraint(value):
			raise ValueError("value did not pass the constraint")
		return value

	def __get__(self, obj, oftype=None):
		"""Property hook for attribute retrieval.

		Unit uses its __key__ to access the correct data row from obj.
		"""
		if obj is None: return self
		try:
			return unbottle(obj.__database__[`obj.__path__+(self.__key__,)`])
		except KeyError:
			return self.default()

	def __set__(self, obj, value):
		"""Property hook for attribute assignment.

		Unit uses its __key__ to store value in the correct data row in obj.
		"""
		obj.__database__[`obj.__path__+(self.__key__,)`] = \
												bottle(self.validate(value))
		obj.__unitkeys__.add_key(self.__key__)

	def __delete__(self, obj):
		"""Property hook for attribute deletion.

		Unit uses its __key__ to delete the correct data row in obj.
		"""
		try:
			del obj.__database__[`obj.__path__+(self.__key__,)`]
		except KeyError:
			return
		obj.__unitkeys__.rem_key(self.__key__)


class Definition(type):
	"""Class definition (container for units).

	The Definition class is the metaclass for Obj. This allows one to build
	classes with typed attributes rapidly with very little overhead:

		>>> class Person(Obj):
		...		name = Text()
		...		age = Int()
		...
		>>> p = Person()
		>>> p.name = "Mr. Bubbles"
		>>> "name" in vars(p) # the name attribute is in the class definition
		0
		>>> print p.name # and its value is stored in the database
		Mr. Bubbles
		>>> p.__unitkeys__ # nonzero units stored in __unitkeys__
		['name']
		>>> p.age = 10
		>>> p.__unitkeys__
		['name', 'age']
		>>> del p.name, p.age
		>>> p.__unitkeys__
		[]
	"""
	def __init__(self, name, bases, dict):
		"""Constructor.

		Overloaded to handle Unit instances in the class' initial
		namespace.

		Arguments:

			name    --  the class name at declaration time.

			bases   --  the base classes of the new class.

			dict    --  the class's namespace at declaration time.
		"""
		type.__init__(self, name, bases, {})
		for key, val in dict.iteritems():
			setattr(self, key, val)

	def __setattr__(self, name, value):
		if isinstance(value, Unit):
			value.__key__ = name
		object.__setattr__(self, name, value)


class Obj(Node):
	"""Persistent object class.

	Assigning units to Obj subclass attributes makes those attributes
	persistent on instances of that Obj subclass. The reason for this is that
	units intercept operations on their namesake attribute and instead of
	storing the attribute's value in the instance's namespace, it is stored
	in the database.

	For those who have designed tables in SQL, this will feel very much like a
	python SQL with the added bonus of adding methods to your tables::

		>>> class Client(Obj):
		...		name = Text(80)
		...		address = Text(100)
		...		phone = Text(25)
		...		def __init__(self, name="", address="", phone=""):
		...			"Constructor"
		...			if name: self.name = name
		...			if address: self.address = address
		...			if phone: self.phone = phone
		...		def __repr__(self):
		...			"Python hook for visual representation of data"
		...			return "Name:%s\\nAddress:%s\\nPhone:%s"%(
		...					self.name, self.address, self.phone)
		...
		>>> myclient = Client("My Client", "123 Main Str.", "(555)555-5555")
		>>>	print myclient
		Name: My Client
		Address: 123 Main Str.
		Phone: (555)555-5555
	"""
	__metaclass__ = Definition
	def __init__(self, copyfrom=None, **keywords):
		"""Constructor.

		Arguments:

			copyfrom	--	if specified, it is the initial argument for
							the __copyfrom__ method.
		"""
		Node.__init__(self, **keywords)
		self.__unitkeys__ = Keyspace()
		if copyfrom is not None:
			self.__copyfrom__(copyfrom)

	def __moveto__(self, database, path=(), parent=None):
		"""Move node to new location.

		Overloaded to take into account non-empty unit rows.
		"""
		for key in self.__unitkeys__:
			database[`path+(key,)`] = self.__database__[`self.__path__+(key,)`]
		Node.__moveto__(self, database, path=(), parent=None)

	def __copyfrom__(self, other):
		"""Make `self` a copy of `other`.

		Overloaded to take into account non-empty unit rows
		"""
		Node.__copyfrom__(self, other)
		for key in other.__unitkeys__:
			self.__database__[`self.__path__+(key,)`] = \
									other.__database__[`other.__path__+(key,)`]

	def __clear__(self):
		"""Recursively delete all data stored under this node.

		Overloaded to take into account non-empty unit rows
		"""
		Node.__clear__(self)
		for key in self.__unitkeys__:
			del self.__database__[`self.__path__+(key,)`]


class Data(Unit):
	"""Storage for string type (8 bit strings).

	An alias for Data is String.
	"""
	def default(self):
		"""Default value to return if the database contains no data in this
		unit's row.

		Return empty string.
		"""
		return ""

	def validate(self, value):
		"""Validate value

		Overloaded to convert data to str if value isn't a StringType.
		"""
		value = Unit.validate(self, value)
		if value is not str:
			return str(value)
		return value

	def __get__(self, obj, oftype=None):
		"""Property hook for attribute retrieval.

		Unit uses its __key__ to access the correct data row from obj.
		"""
		if obj is None: return self
		try:
			return obj.__database__[`obj.__path__+(self.__key__,)`]
		except KeyError:
			return self.default()

	def __set__(self, obj, value):
		"""Property hook for attribute assignment.

		Unit uses its __key__ to store value in the correct data row in obj.
		"""
		obj.__database__[`obj.__path__+(self.__key__,)`] = self.validate(value)
		obj.__unitkeys__.add_key(self.__key__)


String = Data


class Text(Unit):
	"""Storage for unicode text.

	An alias for Text is Unicode.
	"""
	def default(self):
		"""Default value to return if the database contains no data in this
		unit's row.

		Return empty unicode string.
		"""
		return u""

	def validate(self, value):
		"""Validate value.

		Overloaded to convert value to unicode if it isn't UnicodeType.
		"""
		value =  Unit.validate(self, value)
		if type(value) is not unicode:
			return unicode(value)
		return value


Unicode = Text


class Int(Unit):
	"""Storage for IntType.
	"""
	def default(self):
		"""Default value to return if the database contains no data in this
		unit's row.

		Return zero (0).
		"""
		return 0

	def validate(self, value):
		"""Validate value.

		Overloaded to convert value to int() if it isn't IntType.
		"""
		value = Unit.validate(self, value)
		if type(value) is not int:
			return int(value)
		return value


class Long(Int):
	"""Storage for LongType.
	"""
	def default(self):
		"""Default value to return if the database contains no data in this
		unit's row.

		Return zero (0L).
		"""
		return 0L

	def validate(self, value):
		"""Validate value.

		Overloaded to convert value to long() if it isn't LongType.
		"""
		value =  Unit.validate(self, value)
		if type(value) is not long:
			return long(Fvalue)
		return value


class Float(Unit):
	"""Storage for a floating point number.
	"""

	def default(self):
		"""Default value to return if the database contains no data in this
		unit's row.

		Return floating point zero (0.0).
		"""
		return 0.0

	def validate(self, value):
		"""Validate value.

		Overloaded to convert value to unicode if it isn't UnicodeType.
		"""
		value =  Unit.validate(self, value)
		if type(value) is not float:
			return float(value)
		return value


class Type(Unit):
	"""Storage for a class or type reference.
	"""
	def validate(self, value):
		"""Validate value.

		raises ValueError if value is neither a class nor a type.
		"""
		if not isinstance(value, type) or type(value) is ClassType:
			raise TypeError("value must be of class type or instance of type")
		return Unit.validate(self, value)


class Link(Unit):
	"""Reference to another node in the same oops filesystem.
	"""
	def validate(self, value):
		"""Validate value.

		raises ValueError if value is not an instance of a Node subclass.
		"""
		value = Unit.validate(self, value)
		if not isinstance(value, Node):
			raise ValueError("value must be a node")
		return value

	def __set__(self, obj, value):
		"""Property value assignment.

		Overloaded to verify the node is in the same oops filesystem.

		Stores the node's path.
		"""
		if value.__database__ is not obj.__database__:
			raise RuntimeError("value must be in the same node tree as parent "
																		"node")
		obj.__database__[`obj.__path__+(self.__key__,)`] = \
												bottle(self.validate(value))
		obj.__unitkeys__.add_key(self.__key__)

	def __get__(self, obj, oftype=None):
		"""Property value retrieval.

		Overloaded to fetch the node using the stored path.
		"""
		addr = Unit.__get__(self, obj, oftype)
		if addr is None:
			return None
		root = obj
		for i in obj.__path__:
			root = root.__parent__()
		node = root
		for i in addr:
			node = vars(node)[i]
		return node


class Dict(dict, Obj):
	"""Persistent dictionary.

	Important attributes:

		__dictkeys__    --  a keyspace for the dictionary's items.
	"""
	def __init__(self, initval=None, **keywords):
		"""Constructor.

		Accepted arguments:

			initval --  initial value for the dictionary. Calls __setval__.

		"""
		Obj.__init__(self, **keywords)
		self.__dictkeys__ = Keyspace()
		if initval is not None:
			self.__setval__(initval)

	def __moveto__(self, database, path=(), parent=None):
		"""Move the node to a new location.

		Overloaded to copy over dictionary items and keyspace.
		"""
		for key in self.__dictkeys__:
			database[`path+(`key`,)`] = \
									self.__database__[`self.__path__+(`key`,)`]
		Obj.__moveto__(self, database, path, parent)

	def __setval__(self, value):
		"""Set the node's value.

		Accepts instances the python builtin dict and derived classes.
		"""
		if not isinstance(value, dict):
			raise TypeError("value must be instance of dict")
		if isinstance(value, Dict):
			self.__copyfrom__(value)
		else:
			self.clear()
			self.update(value)

	def __copyfrom__(self, other):
		"""Make `self` a copy of `other`.

		Overloaded to copy over dictionary items and keyspace.
		"""
		Obj.__copyfrom__(self, other)
		for key in other.__dictkeys__:
			self.__database__[`self.__path__+(`key`,)`] = \
								other.__database__[`other.__path__+(`key`,)`]

	def __clear__(self):
		"""Delete all data associated to this node and all contained subnodes.

		Overloaded to delete all dict items.
		"""
		self.clear()
		Obj.__clear__(self)

	def __len__(self):
		"""Return the number of dictionary items.
		"""
		return len(self.__dictkeys__)

	def __iter__(self):
		"""Return a iterator of the dictionary's keys
		"""
		return self.iterkeys()

	def __nonzero__(self):
		"""Tests for emptyness.

		Return true (1) if the dictionary has 1 or more items.
		"""
		return len(self)>0 or Obj.__nonzero__(self)

	def __repr__(self):
		"""String representation of a Dict instance.

		Overloaded to better represent the Dict instance's value.
		"""
		return "<%s path=%s items=%r>"%(type(self).__name__,
										self.__path__,
										self.copy())

	def __cmp__(self, other):
		"""Comparison operator.

		Overloaded to more adaquately compare Dict to other values.
		"""
		if isinstance(other, Dict):
			return cmp(self.copy(), other.copy())
		return cmp(self.copy(), other)

	def __eq__(self, other):
		"""Test for equality.
		"""
		if isinstance(other, Dict):
			return self.copy()==other.copy()
		return self.copy()==other

	def __ne__(self, other):
		"""Test for inequality.
		"""
		if isinstance(other, Dict):
			return self.copy()!=other.copy()
		return self.copy()!=other

	def __gt__(self, other):
		"""Test for greater than.
		"""
		if isinstance(other, Dict):
			return self.copy()>other.copy()
		return self.copy()>other

	def __ge__(self, other):
		"""Test for greater of equal to.
		"""
		if isinstance(other, Dict):
			return self.copy()>=other.copy()
		return self.copy()>=other

	def __lt__(self, other):
		"""Test for lesser than.
		"""
		if isinstance(other, Dict):
			return self.copy()<other.copy()
		return self.copy()<other

	def __le__(self, other):
		"""Test for lesser or equal to.
		"""
		if isinstance(other, Dict):
			return self.copy()<=other.copy()
		return self.copy()<=other

	def __getitem__(self, key):
		"""Index operator for dictionary item retrieval.

		If no value is associated to `key`, raises KeyError.
		"""
		if `key` in vars(self):
			return vars(self)[`key`]
		try:
			value = unbottle(self.__database__[`self.__path__+(`key`,)`])
		except KeyError:
			raise KeyError(key)
		if isinstance(value, Node):
			value.__bindto__(self.__database__, self.__path__+(`key`,), self)
			vars(self)[`key`] = value
		return value

	def __setitem__(self, key, value):
		"""Index operator for dictionary item assignment.

		If __oftype__ is not None & type(value) is not the same as __oftype__
		this raises a TypeError.
		"""
		if `key` in vars(self):
			vars(self)[`key`].__clear__()
		if isinstance(value, Node):
			if value.__parent__() is None:
				if value:
					value.__moveto__(self.__database__,
												self.__path__+(`key`,), self)
				else:
					value.__bindto__(self.__database__,
												self.__path__+(`key`,), self)
			else:
				value = type(value)(copyfrom=value,
				datasource=(self.__database__, self.__path__+(`key`,), self))
			vars(self)[`key`] = value
		try:
			self.__database__[`self.__path__+(`key`,)`] = bottle(value)
		except:
			print ``self.__path__+(`key`,)``
			print `bottle(value)`
			raise
		self.__dictkeys__.add_key(key)

	def __delitem__(self, key):
		"""Index operator for dictionary item deletion.

		If no value is associated to `key`, raises KeyError.
		"""
		if `key` in vars(self):
			vars(self)[`key`].__clear__()
			del vars(self)[`key`]
		self.__dictkeys__.rem_key(key)
		del self.__database__[`self.__path__+(`key`,)`]

	def get(self, key, default):
		"""Return value associated to `key`. If there is no data, return
		`default`.
		"""
		try:
			return self[key]
		except KeyError:
			return default

	def setdefault(self, key, default):
		"""If no data is associated to `key`, associate value `default` to it.
		"""
		if key not in self:
			self[key] = default

	def popitem(self, *args):
		"""Return a (non-specific) key-value pair.

		If the dictionary is empty, this raises a KeyError.
		"""
		key = self.__dictkeys__[-1]
		val = self[key]
		if isinstance(val, Node):
			val.__moveto__({})
		del self[key]
		return key, val

	def update(self, extension):
		"""Update the current dictionary with values `from extension`.

		extension can be either a python dictionary, an instance of UserDict or
		an instance of Dict.
		"""
		for key in extension:
			self[key] = extension[key]

	def clear(self):
		"""Deletes all dictionary items.
		"""
		for key in self.__dictkeys__:
			del self.__database__[`self.__path__+(`key`,)`]
			if `key` in vars(self):
				vars(self)[`key`].__clear__()
				del vars(self)[`key`]
		self.__dictkeys__.__clear__()

	def copy(self):
		"""Returns a python dictionary filled with all the dictionary items
		stored in `self`.
		"""
		copy = {}
		for key in self:
			copy[key] = self[key]
		return copy

	def __contains__(self, key):
		"""Test if self has data associated to `key`.
		"""
		return self.has_key(key)

	def has_key(self, key):
		"""Return true (1) is self has data associated to `key`. Return false
		(0) otherwise.
		"""
		return key in self.__dictkeys__

	def keys(self):
		"""Return a list of all the keys that have data associated to them.
		"""
		return self.__dictkeys__[:]

	def iterkeys(self):
		"""Return an iterator that iterates over all the keys of this
		dictionary.
		"""
		for key in self.__dictkeys__:
			yield key

	def items(self):
		"""Returns a list of key-value pairs represented by tuples in the
		format (key, value).
		"""
		return [(key, self[key]) for key in self.__dictkeys__]

	def iteritems(self):
		"""Returns an iterator that iterates over all the key-value pairs in
		this dictionary.
		"""
		for key in self.__dictkeys__:
			yield (key, self[key])

	def values(self):
		"""Returns a list of all the values stored in this dictionary.
		"""
		return [self[key] for key in self.__dictkeys__]

	def itervalues(self):
		"""Returns an iterator that iterates over all the values stored in this
		dictionary.
		"""
		for key in self.__dictkeys__:
			return self[key]


class List(list, Obj):
	"""Sequential container node.
	"""
	autonum = Long()
	lookup = Data()
	def __init__(self, initval=None, **keywords):
		"""Constructor.

		Arguments:

			initval --  optional initial value. Can be a regular python list,
						an instance of UserList or another instance of List.

		Optional keyword arguments (in addition to those for Node):

			oftype  --  initial value for __oftype__.

		"""
		list.__init__(self)
		Obj.__init__(self, **keywords)
		if initval is not None:
			self.__setval__(initval)

	def __bindto__(self, database, path=(), parent=None):
		"""Bind the list to a new storage location.

		Overloaded to load the lookup table from disk.
		"""
		Obj.__bindto__(self, database, path, parent)
		list.__setslice__(self, 0, len(self), self.lookup.split())
		if not hasattr(self, "__listnodes__"):
			self.__listnodes__ = Keyspace()
		for i in self.__listnodes__:
			node = self[i]
			node.__bindto__(self.__database__, self.__path__+(i,), self)
			vars(self)[i] = node

	def __moveto__(self, database, path=(), parent=None):
		"""Move the list to a new storage location.

		Overloaded to copy list items to the new location.
		"""
		for key in list.__iter__(self):
			database[`path+(`key`,)`] = \
									self.__database__[`self.__path__+(`key`,)`]
		for key in self.__listnodes__:
			vars(self)[key].__moveto__(self.__database__,
													self.__path__+(key,), self)
		Obj.__moveto__(self, database, path, parent)

	def __setval__(self, value):
		"""Data assignment for list attributes.

		Accepts instances of list.
		"""
		if not isinstance(value, list):
			raise TypeError("value must be an instance of list")
		if isinstance(value, List):
			self.__copyfrom__(value)
		else:
			self[:] = value

	def __copyfrom__(self, other):
		"""Copy data from other.

		Overloaded to copy over other's list items as well as its lookup.
		"""
		Obj.__copyfrom__(self, other)
		list.__setslice__(self, 0, len(self), other)
		for key in list.__iter__(other):
			self.__database__[`self.__path__+(key,)`] = \
									other.__database__[`other.__path__+(key,)`]
		for key in other.__listnodes__:
			node = vars(other)[key]
			vars(self)[key] = type(node)(copyfrom=node,
				datasource=(self.__database__, self.__path__+(key,), self))

	def __clear__(self):
		"""Empty the List.

		Overloaded to delete the list items.
		"""
		for key in self.__listnodes__:
			vars(self)[key].__clear__()
			del vars(self)[key]
		for key in list.__iter__(self):
			del self.__database__[`self.__path__+(key,)`]
		Obj.__clear__(self)

	def __repr__(self):
		"""String representation.

		Overloaded to better represent a List instance.
		"""
		return "<%s path=%r items=%r>" % (
								type(self).__name__, self.__path__, self[:])

	def __cmp__(self, other):
		"""Comparison operator.

		Overloaded to more adaquately compare Dict to other values.
		"""
		if isinstance(other, List):
			return cmp(self[:], other[:])
		return cmp(self[:], other)

	def __eq__(self, other):
		"""Test for equality.
		"""
		if isinstance(other, List):
			return self[:]==other[:]
		return self[:]==other

	def __ne__(self, other):
		"""Test for inequality.
		"""
		if isinstance(other, List):
			return self[:]!=other[:]
		return self[:]!=other

	def __gt__(self, other):
		"""Test for greater than.
		"""
		if isinstance(other, List):
			return self[:]>other[:]
		return self[:]>other

	def __ge__(self, other):
		"""Test for greater of equal to.
		"""
		if isinstance(other, List):
			return self[:]>=other[:]
		return self[:]>=other

	def __lt__(self, other):
		"""Test for lesser than.
		"""
		if isinstance(other, List):
			return self[:]<other[:]
		return self[:]<other

	def __le__(self, other):
		"""Test for lesser or equal to.
		"""
		if isinstance(other, List):
			return self[:]<=other[:]
		return self[:]<=other

	def __add__(self, other):
		"""invoked by self+other

			other   --  must be a list type, an instance of UserList or another
						instance of List.
		"""
		if isinstance(other, List):
			return self[:]+other[:]
		return self[:]+other

	def __radd__(self, other):
		"""Invoked by other+self.

			other   --  must be a list type, an instance of UserList or another
						instance of List.
		"""
		if isinstance(other, List):
			return other[:]+self[:]
		return other+self[:]

	def __iadd__(self, other):
		"""Invoked by self+=other

			other   --  must be a list type, an instance of UserList or another
						instance of List.
		"""
		self.extend(other)
		return self

	def __mul__(self, other):
		"""Invoked by self*other

			other   --  must be an integer
		"""
		return self[:]*other

	__rmul__=__mul__

	def __imul__(self, other):
		"""Invoked by self*=other

			other   --  must be an integer
		"""
		self[:] *= other
		return self

	def count(self, value):
		"""Returns the number of times `value` appears in the list.

			value   --  value to count
		"""
		count = 0
		for i in self:
			if i==value:
				count+=1
		return count

	def index(self, value):
		"""Return the index of the first occurence of `value`.

			value   --  value for which to search.

		If the item is not found, raises an IndexError.
		"""
		for i in range(len(self)):
			if self[i]==value:
				return i

	def remove(self, value):
		"""Locate and remove item from the list.

			value   --  item to locate and remove.

		If the item is not found, raises IndexError.
		"""
		del self[self.index(value)]

	def append(self, value):
		"""Inserts an item at the end of the list.

			value   --  item to insert.

		If self.__oftype__ is not None and `value` is not of that type, raises
		a TypeError.
		"""
		# get new key
		key = `self.autonum`
		self.autonum += 1
		while list.__contains__(self, `self.autonum`):
			self.autonum += 1
		# if Node bind to self
		if isinstance(value, Node):
			if value.__parent__() is None:
				value.__moveto__(self.__database__, self.__path__+(key,), self)
			else:
				value = type(value)(copyfrom=value,
					datasource=(self.__database__, self.__path__+(key,), self))
			vars(self)[key] = value
		# save changes
		list.append(self, key)
		self.lookup = " ".join(self)
		self.__database__[`self.__path__+(key,)`] = bottle(value)

	def insert(self, index, value):
		"""Insert an item at a specific position in the list.

			index   --  position in the list at which to insert the new item.

			value   --  item to insert.

		If self.__oftype__ is not None and `value` is not of that type, raises
		a TypeError.
		"""
		# get new key
		key = `self.autonum`
		self.autonum += 1
		while list.__contains__(self, `self.autonum`):
			self.autonum += 1
		# if Node bind to self
		if isinstance(value, Node):
			if value.__parent__() is None:
				value.__moveto__(self.__database__, self.__path__+(key,), self)
			else:
				value = type(value)(copyfrom=value,
					datasource=(self.__database__, self.__path__+(key,), self))
			vars(self)[key] = value
		# save changes
		list.insert(self, index, key)
		self.lookup = " ".join(self)
		self.__database__[`self.__path__+(key,)`] = bottle(value)

	def pop(self, index=-1):
		"""Remove and return the list item located at position `index`.

			index   --  position of the item you wish to fetch and remove.
		"""
		key = list.pop(self, index)
		del self.__database__[`self.__path__+(key,)`]
		try:
			self.__listnodes__.rem_key(key)
		except KeyError:
			pass
		else:
			vars(self)[key].__clear__()
			del vars(self)[key]

	def __getitem__(self, index):
		"""Retrieve item from a specific position in the list.

			index   -- position in the list at which to fetch the item.

		If index goes beyond the bounds of the list, this raises an IndexError.
		"""
		key = list.__getitem__(self, index)
		if key in self.__listnodes__:
			return vars(self)[key]
		return unbottle(self.__database__[`self.__path__+(key,)`])

	def __getitem__(self, index):
		"""Retrive item at index.
		"""
		key = list.__getitem__(self, index)
		if key in self.__listnodes__:
			return vars(self)[key]
		return unbottle(self.__database__[`self.__path__+(key,)`])

	def __getslice__(self, index, until):
		"""Retrieve a slice of the list.

			index   --  beginning of the slice

			until   --  end of the slice (non-inclusive)
		"""
		keys = list.__getslice__(self, index, until)
		retval = []
		for key in keys:
			try:
				retval.append(vars(self)[key])
			except KeyError:
				retval.append(
				unbottle(self.__database__[`self.__path__+(key,)`]))
		return retval

	def __setitem__(self, index, value):
		"""Assign a list item to a specific position in the list.

			index   --  position in the list at which to assign the item.

		If self.__oftype__ is not None and `value` is not of that type, raises
		a TypeError.
		"""
		# get new key
		key = `self.autonum`
		self.autonum = self.autonum + 1
		while list.__contains__(self, `self.autonum`):
			print "autonum %d" % self.autonum
			self.autonum = self.autonum + 1
		# if Node bind to self
		if isinstance(value, Node):
			if value.__parent__() is None:
				value.__moveto__(self.__database__, self.__path__+(key,), self)
			else:
				value = type(value)(copyfrom=value,
					datasource=(self.__database__, self.__path__+(key,), self))
			vars(self)[key] = value
		# remove old resident of this index
		oldkey = list.__getitem__(self, index)
		try:
			self.__listnodes__.rem_key(oldkey)
		except KeyError:
			pass
		else:
			vars(self)[oldkey].__clear__()
			del vars(self)[oldkey]
		del self.__database__[`self.__path__+(oldkey,)`]
		# syncronize changes to database
		list.__setitem__(self, index, key)
		self.lookup = " ".join(self)
		self.__database__[`self.__path__+(key,)`] = bottle(value)

	def extend(self, extension):
		"""Extend this list with another list.

			extension   --  extension to append to the list.
		"""
		for i in xrange(len(extension)):
			key = `self.autonum`
			self.autonum += 1
			while list.__contains__(self, `self.autonum`):
				self.autonum += 1
			value = extension[i]
			if isinstance(value, Node):
				if value.__parent__() is None:
					value.__moveto__(self.__database__,
													self.__path__+(key,), self)
				else:
					value = type(value)(copyfrom=value,
					datasource=(self.__database__, self.__path__+(key,), self))
				vars(self)[key] = value
			self.__database__[`self.__path__+(key,)`] = bottle(value)
			list.append(self, key)
		self.lookup = " ".join(self)

	def __setslice__(self, index, until, replacement):
		"""Replace slice with another list.

			index   --  beginning of the slice

			until   -- end of the slice (non-inclusive)

			replacement --  replacement list.

		If self.__oftype__ is not None and the items in `replacement` are not
		of that type, raises a TypeError.
		"""
		keys = []
		for i in xrange(len(replacement)):
			key = `self.autonum`
			self.autonum += 1
			while list.__contains__(self, `self.autonum`):
				self.autonum += 1
			value = replacement[i]
			if isinstance(value, Node):
				if value.__parent__() is None:
					value.__moveto__(self.__database__,
													self.__path__+(key,), self)
				else:
					value = type(value)(copyfrom=value,
					datasource=(self.__database__, self.__path__+(key,), self))
				vars(self)[key] = value
			self.__database__[`self.__path__+(key,)`] = bottle(value)
			keys.append(key)
		minkey = self.autonum
		for key in list.__getslice__(self, index, until):
			try:
				self.__listnodes__.rem_key(key)
			except KeyError:
				pass
			else:
				vars(self)[key].__clear__()
				del vars(self)[key]
			del self.__database__[`self.__path__+(key,)`]
		list.__setslice__(self, index, until, keys)
		self.lookup = " ".join(self)

	def __delitem__(self, index):
		"""Delete an item from the list located at a specific position.

			index   --  position in the list at which to delete the item.

		If index goes beyond the bounds of the list, this raises an IndexError.
		"""
		key = list.pop(self, index)
		try:
			self.__listnodes__.rem_key(key)
		except KeyError:
			pass
		else:
			vars(self)[key].__clear__()
			del vars(self)[key]
		del self.__database__[`self.__path__+(key,)`]
		self.lookup = " ".join(self)

	def __delslice__(self, index, until):
		"""Delete a slice from the list.

			index   --  position at which to start deleting

			until   --  position at which to stop deleting
		"""
		keys = list.__getslice__(self, index, until)
		for key in keys:
			try:
				self.__listnodes__.rem_key(key)
			except KeyError:
				pass
			else:
				vars(self)[key].__clear__()
				del vars(self)[key]
			del self.__database__[`self.__path__+(key,)`]
		list.__delslice__(self, index, until)
		self.lookup = " ".join(self)

	def reverse(self):
		"""Reverse the order of the list.

		The first item becomes the last, the second becomes the before last,
		the last becomes the first, etc.
		"""
		list.reverse(self)
		self.lookup = " ".join(self)

	def sort(self, *args):
		"""Sort the list by value of its items
		"""
		cp =[(self[x], list.__getitem__(self, x)) for x in xrange(len(self))]
		cp.sort(*args)
		list.__setslice__(self, 0, len(self),
											[x[1] for x in xrange(len(self))])
		self.lookup = " ".join(self)

